import axios, {
  AxiosResponse,
  AxiosError
} from "../../node_modules/axios/index"; // Don't worry about red lines here ...

document.getElementById('getRequestButton1').addEventListener("click", performGetRequest1Async);
document.getElementById('getRequestButton2').addEventListener("click", performGetRequest2);
document.getElementById('todoInputForm').addEventListener('submit', performPostRequest);
document.getElementById('clearButton1').addEventListener("click", clearOutput1);
document.getElementById('clearButton2').addEventListener("click", clearOutput2);
document.getElementById('clearButton3').addEventListener("click", clearOutput3);


function performGetRequest1() {
    var resultElement = document.getElementById('getResult1');
    resultElement.innerHTML = '';
    
    axios.get('http://jsonplaceholder.typicode.com/todos')
      .then(function (response) {
        resultElement.innerHTML = generateSuccessHTMLOutput(response);
      })
      .catch(function (error) {
        resultElement.innerHTML = generateErrorHTMLOutput(error);
      });   
  }

async function performGetRequest1Async(){ 
  let resultElement = document.getElementById('getResult1');
  resultElement.innerHTML = '';

  try {
      let response = await axios.get<AxiosResponse>('http://jsonplaceholder.typicode.com/todos');
      resultElement.innerHTML = generateSuccessHTMLOutput(response);
  } 
  catch (error) {
      let axiosError: AxiosError = error;
      resultElement.innerHTML = generateErrorHTMLOutput(axiosError);
  }

}
  
  function performGetRequest2() {
    var resultElement = document.getElementById('getResult2');
    var todoId : HTMLInputElement = <HTMLInputElement>document.getElementById('todoId');
    resultElement.innerHTML = '';
    
    axios.get('http://jsonplaceholder.typicode.com/todos', {
      params: {
        id: todoId.value
      }
    })
    .then(function (response) {
      console.log(response);
      resultElement.innerHTML = generateSuccessHTMLOutput(response);
    })
    .catch(function (error) {
        resultElement.innerHTML = generateErrorHTMLOutput(error);
    });
  }
  
  function performPostRequest(e : Event) {
    var resultElement = document.getElementById('postResult');
    var todoTitle : HTMLInputElement = <HTMLInputElement>document.getElementById('todoTitle');
    resultElement.innerHTML = '';
    
    axios.post('http://jsonplaceholder.typicode.com/todos', {
      userId: '1',
      title: todoTitle.value,
      completed: false
    })
    .then(function (response) {
      resultElement.innerHTML = generateSuccessHTMLOutput(response);
    })
    .catch(function (error) {
      resultElement.innerHTML = generateErrorHTMLOutput(error);
    });
    
    e.preventDefault();
  }

 
  function clearOutput1(){
    var resultElement = document.getElementById('getResult1');
    resultElement.innerHTML = '';
  }
  
  function clearOutput2(){
    var resultElement = document.getElementById('getResult2');
    resultElement.innerHTML = '';
  }
  
  function clearOutput3(){
    var resultElement = document.getElementById('postResult');
    resultElement.innerHTML = '';
  }

  function generateSuccessHTMLOutput(response : AxiosResponse) {
    return  '<h4>Result</h4>' + 
            '<h5>Status:</h5> ' + 
            '<pre>' + response.status + ' ' + response.statusText + '</pre>' +
            '<h5>Headers:</h5>' + 
            '<pre>' + JSON.stringify(response.headers, null, '\t') + '</pre>' + 
            '<h5>Data:</h5>' + 
            '<pre>' + JSON.stringify(response.data, null, '\t') + '</pre>'; 
  }

  function generateErrorHTMLOutput(error : AxiosError) {
    return  '<h4>Result</h4>' + 
            '<h5>Message:</h5> ' + 
            '<pre>' + error.message + '</pre>' +
            '<h5>Status:</h5> ' + 
            '<pre>' + error.response.status + ' ' + error.response.statusText + '</pre>' +
            '<h5>Headers:</h5>' + 
            '<pre>' + JSON.stringify(error.response.headers, null, '\t') + '</pre>' + 
            '<h5>Data:</h5>' + 
            '<pre>' + JSON.stringify(error.response.data, null, '\t') + '</pre>'; 
  }